import { useEffect, useRef } from 'react';

export type DialogProps = {
  onClose: () => void;
  open: boolean;
  children?: React.ReactNode;
  className?: string;
};

export const Dialog = (props: DialogProps) => {
  const ref = useRef<HTMLDialogElement>(null);

  useEffect(() => {
    if (props.open) ref.current?.showModal();
    else ref.current?.close();
  }, [props.open]);

  return (
    <dialog data-modal className={'modal'} ref={ref} onClose={() => props.onClose()}>
      <form method="dialog" className={'modal-box card flex gap-4 ' + (props?.className ? props?.className : '')}>
        {props.children}
      </form>
      <form method="dialog" className="modal-backdrop">
        <button>close</button>
      </form>
    </dialog>
  );
};
