import React, { useState, useEffect } from 'react';
import { EncodingSelector } from './selectors';
import { useGraphQueryResultMeta } from '@graphpolaris/shared/lib/data-access';
import { ElementType } from './encodings.types';
import { DimensionType } from '@graphpolaris/shared/lib/schema';

type Props = {
  selectorType: keyof typeof EncodingSelector;
  marking: any;
  elementType: ElementType;
  onChange: (val: any) => void;
  accessorPath: string;
};

export default function Selector({ selectorType, elementType, marking, onChange, accessorPath }: Props) {
  const graphStatistics = useGraphQueryResultMeta();
  const SelectorComponent = EncodingSelector[selectorType];
  const [statistics, setStatistics] = useState<any>();
  const [dimension, setDimension] = useState<DimensionType>('categorical');

  useEffect(() => {
    const group = graphStatistics[elementType === 'node' ? 'nodes' : 'edges'];

    const combinedValues = Array.from(
      new Set(
        Object.keys(group.types).flatMap((label) => {
          const obj = group.types[label].attributes[accessorPath];
          const { dimension } = obj;
          setDimension(dimension);

          if (dimension === 'categorical') {
            const { values } = obj;
            return values || [];
          } else if (dimension === 'numerical') {
            console.log(obj);
            const { min, max } = obj.statistics;
            return [min, max];
          } else if (dimension === 'temporal') {
            const { minDate, maxDate } = obj.statistics;
            return [minDate, maxDate];
          } else if (dimension === 'spatial') {
            const { boundingBox } = obj.statistics;
            return boundingBox;
          }
          return [];
        }),
      ),
    );

    setStatistics(combinedValues);
  }, [accessorPath]);

  return <SelectorComponent marking={marking} onChange={onChange} dimension={dimension} statistics={statistics} />;
}
