import React from 'react';
import { Meta } from '@storybook/react';
import { VisualizationPanel } from '../../panel';
import {
  assignNewGraphQueryResult,
  graphQueryResultSlice,
  querybuilderSlice,
  schemaSlice,
  visualizationSlice,
} from '../../../data-access/store';
import { configureStore } from '@reduxjs/toolkit';
import { Provider } from 'react-redux';
import { big2ndChamberQueryResult, smallFlightsQueryResults, mockLargeQueryResults } from '../../../mock-data';
import { setActiveVisualization } from '@graphpolaris/shared/lib/data-access/store/visualizationSlice';

const Component: Meta<typeof VisualizationPanel> = {
  title: 'Visualizations/NodeLinkVis',
  component: VisualizationPanel,
  decorators: [
    (story) => (
      <Provider store={Mockstore}>
        <div
          style={{
            width: '100%',
            height: '100vh',
          }}
        >
          {story()}
        </div>
      </Provider>
    ),
  ],
};

const Mockstore = configureStore({
  reducer: {
    schema: schemaSlice.reducer,
    graphQueryResult: graphQueryResultSlice.reducer,
    visualize: visualizationSlice.reducer,
    querybuilder: querybuilderSlice.reducer,
  },
});

export const TestWithData = {
  layout: 'fullscreen',
  play: async () => {
    const dispatch = Mockstore.dispatch;

    dispatch(
      assignNewGraphQueryResult({
        queryID: '1',
        result: {
          type: 'nodelink',
          payload: {
            nodes: [
              { id: 'agent/007', attributes: { name: 'Daniel Craig' } },
              { id: 'villain', attributes: { name: 'Le Chiffre' } },
            ],
            edges: [{ id: 'escape/escape', from: 'agent/007', to: 'villain', attributes: { name: 'Escape' } }],
          },
        },
      }),
    );
    dispatch(setActiveVisualization('NodeLinkVis'));
  },
};

export const TestWithNoData = {
  args: { loading: false },
  play: async () => {
    const dispatch = Mockstore.dispatch;
    dispatch(
      assignNewGraphQueryResult({
        queryID: '1',
        result: {
          type: 'nodelink',
          payload: {
            nodes: [],
            edges: [],
          },
        },
      }),
    );
    dispatch(setActiveVisualization('NodeLinkVis'));
  },
};

export const TestWithBig2ndChamber = {
  play: async () => {
    const dispatch = Mockstore.dispatch;
    dispatch(assignNewGraphQueryResult({ queryID: '1', result: { type: 'nodelink', payload: big2ndChamberQueryResult } }));
    dispatch(setActiveVisualization('NodeLinkVis'));
  },
};

export const TestWithSmallFlights = {
  args: { loading: false },
  play: async () => {
    const dispatch = Mockstore.dispatch;
    dispatch(assignNewGraphQueryResult({ queryID: '1', result: { type: 'nodelink', payload: smallFlightsQueryResults } }));
    dispatch(setActiveVisualization('NodeLinkVis'));
  },
};

export const TestWithLargeQueryResult = {
  args: { loading: false },
  play: async () => {
    const dispatch = Mockstore.dispatch;
    dispatch(assignNewGraphQueryResult({ queryID: '1', result: { type: 'nodelink', payload: mockLargeQueryResults } }));
    dispatch(setActiveVisualization('NodeLinkVis'));
  },
};

export default Component;
